package com.sziit.G06.controller;

import com.sziit.G06.controller.MyException;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.ObjectOutputStream;

/**
 *控制端(对客户端进行远程)：
 * 1.连接被控端，进行控制。
 * 2.接收从被控端发送过来的屏幕截图，实现在当前窗口的画面回放。
 * 3.把对控制端的操作(键盘事件、鼠标事件)发送到控制端。
 */

public class Controller extends Thread {

    private DataInputStream dis;//读视频流对象
    private ObjectOutputStream ous;//写事件流
    private JLabel la_image = new JLabel();


    /*
     * 显示页面
     */
    public void showUI(String ip,int port,String idname) {

//////发现鼠标不能精准定位的问题：

        //获取当前远程控制屏幕的大小
        Dimension dim=Toolkit.getDefaultToolkit().getScreenSize();
        int w=dim.width;
        int h=dim.height;

        System.out.println("w = " + w);
        System.out.println("h = " + h);

        //获取（被控制端）不包含任务栏的屏幕大小
        GraphicsEnvironment ge=GraphicsEnvironment.getLocalGraphicsEnvironment();
        Rectangle rect=ge.getMaximumWindowBounds();
        int w1=rect.width;
        int h1=rect.height;

        System.out.println("w1 = " + w1);
        System.out.println("h1 = " + h1);

/////解决鼠标不能精准定位的方法：

        //1.设置远程控制页面

        //设置远程控制窗口的标题
        JFrame frame=new JFrame("远程控制");    //-----问题所在：存在任务栏高度差！
//        frame.setSize(1920,1080);
        //设置远程控制窗口标题

        //设置远程控制屏幕大小，整个屏幕高度(包含任务栏高度)
        frame.setSize(w,h);
        //设置远程控制屏幕可见
        frame.setResizable(false);

        frame.setTitle(ip +"    端口:" + port+"    被控制端名字:"+idname);
        //设置被控制端屏幕大小，不包含任务栏高度
        la_image.setSize(w1, h1);
        JLayeredPane jlp = new JLayeredPane();
        jlp.add(la_image);
        frame.setLayeredPane(jlp);
        //设置可见
        frame.setVisible(true);
        //控制台退出模式
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        //设置置顶
        frame.setAlwaysOnTop(true);

//////问题解决！

        //2.监听键盘事件并处理
        frame.addKeyListener(new KeyAdapter(){
            public void keyPressed(KeyEvent e) {
                sendEventObject(e);
            }
            @Override
            public void keyReleased(KeyEvent e) {
                sendEventObject(e);
            }
            @Override
            public void keyTyped(KeyEvent e) {
            }
        });
        //3.监听鼠标转轮事件并处理
        frame.addMouseWheelListener(new MouseWheelListener(){
            public void mouseWheelMoved(MouseWheelEvent e) {
                sendEventObject(e);
            }
        });



        //4.监听鼠标移动
        frame.addMouseMotionListener(new MouseMotionListener(){
            public void mouseDragged(MouseEvent e) {
                sendEventObject(e);
            }
            public void mouseMoved(MouseEvent e) {
                sendEventObject(e);
            }
        });

        //5.监听鼠标按键
        frame.addMouseListener(new MouseListener(){
            public void mouseClicked(MouseEvent e) {
                sendEventObject(e);
            }
            public void mouseEntered(MouseEvent e) {
                sendEventObject(e);
            }
            public void mouseExited(MouseEvent e) {
                sendEventObject(e);
            }
            public void mousePressed(MouseEvent e) {
                sendEventObject(e);
            }
            public void mouseReleased(MouseEvent e) {
                sendEventObject(e);
            }
        });
    }
    //连接服务端
    private void connectToServer(String ip,int port)
            throws Exception{

        java.net.Socket sc=new java.net.Socket(ip,port);
//得到输入流，读取图片数据
        dis=new DataInputStream(sc.getInputStream());
//得到输出流，发送事件对象
        ous=new ObjectOutputStream(sc.getOutputStream());

        if(dis==null||ous==null)throw new MyException("远程控制不接受或未接受被控制。。。");

    }
    //将控制端所侦听到的事件封装成InputEvent类，用对象输出流发送到被控制端。

    //发送事件对象到被控制端
    private void sendEventObject(InputEvent event){
        try{
            ous.writeObject(event);

        }catch(Exception ef){

            ef.printStackTrace();
        }

    }

    public void run(){

        try{
            //读取图片数据
            while(true){
                //图片长度
                int len=dis.readInt();
                //把数据储存在字节数组
                byte[] data=new byte[len];
                //读取数据输出流中的图片数据
                dis.readFully(data);

                //通过输入流，以javax.io.ImagIO转成数组，读取图片数据
                ByteArrayInputStream bins=new ByteArrayInputStream(data);
                BufferedImage image= ImageIO.read(bins);

                //将读到的数据生成为一个图标对象
                ImageIcon ic=new ImageIcon(image);
                Image img = ic.getImage();

                // 2.根据屏幕设定图片的大小
                Toolkit tk = Toolkit.getDefaultToolkit() ;
                Dimension d =tk.getScreenSize();

                int w = d.width;
                int h =d.height;

                //将读取到的图片回放
//                BufferedImage bi = resize(img,1920,1080);
                BufferedImage bi = resize(img,w,h);

                la_image.setIcon(new ImageIcon(bi));

                la_image.repaint();//销掉以前画的背景
            }
        }catch(Exception ef){
            System.out.println("网络故障：无法读出远程图片数据。。。");
            ef.printStackTrace();
        }

    }

    //从被控制端传输的图片在控制端实现回放
    private static BufferedImage resize(Image img, int newW, int newH) {
//设置在控制端回放的图片大小
        int w = img.getWidth(null);
        int h = img.getHeight(null);

        BufferedImage dimg = new BufferedImage(newW, newH,BufferedImage.TYPE_INT_BGR);

        Graphics2D g = dimg.createGraphics();
        //Graphics2D属于Graphics 类，
        // 提供了对几何形状、坐标转换、颜色管理和文本布局更为复杂的控制。
        // 它是用于在 Java(tm) 平台上呈现二维形状、文本和图像的基础类。

        //设置模式
        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);

        //按比例缩放图片
        g.drawImage(img, 0, 0, newW, newH, 0, 0, w, h, null);
        g.dispose();
        return dimg;

    }

    /**
     * 1.创建对象
     * 2.建立显示界面，showUI()实现
     * 3.连接被控端，私有方法connectToServer(ip,port)实现
     * 4.启动线程，start()方法
     *
     * @param
     * @throws Exception
     */

    public static void connect(String ip,int port,String idname) throws Exception{
        //1.创建对象
        Controller ct=new Controller();
        //2.建立显示界面，showUI()实现
        ct.showUI(ip,port,idname);
        //3.连接被控端，私有方法connectToServer(ip,port)实现
        ct.connectToServer(ip,port);
        //4.启动线程，start()方法
        ct.start();
    }

}
