/*
   *Java 实现远程控制:
    1.项目介绍：
           控制端(Controller)、被控端 (RemoteComputer)相当于客户端 ,服务端(Server) 相当于服务端。
        被控端 (RemoteComputer) 打开一个端口通过服务端等待控制端(Controller)来连接，
        Server 实现服务端监视界面,当有客户端请求连接后，开启一个线程负责和它通讯。
        一旦连接成功，被控端(RemoteComputer)不断截取屏幕通过图片传输线程(CaptureThread)发送到控制端(Controller),
        控制端控制的过程实际上就是操作键盘和鼠标的过程，因此控制端会把键盘和鼠标事件
        通过事件读取线程(EventReadThread)发送到被控制端 (RemoteComputer),
        被控制端接收到事件后回放，就起到被控制的效果。
     2.项目意义：
          最终实现实用性：多个客户机通过服务端(Server)实现远程控制。
     3.遇到问题：
        遇到的最大问题：鼠标不能精准的定位？
        如何解决这个问题？
 */

package com.sziit.G06.controller;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;


public class Server extends JFrame {

    /*
     * (服务端)：
     * 实现服务端监视界面,当有客户端请求连接后，开启一个线程负责和它通讯。
     * 接受从被控端发出的请求，根据端口指定对应的线程执行其要求连接的对象与其通讯。
     * */


    private static final long serialVersionUID = 1L;
    private JTextField enterField;
    private JTextArea displayArea;
    private ObjectOutputStream output;
    private ObjectInputStream input;
    private Socket client;
    private String chatServer;
    private String message;

    /*
    生成服务端通讯界面。
    */

    //设置文本框的大小
    public JTextField porttf  =new JTextField(3);
    public JTextField iptf  =new JTextField(10);
    public JTextField idname =new JTextField(10);

    //添加连接与断开按钮
    public JButton jb1 = new JButton("连接");

    public JButton jb2 = new JButton("断开");

    public void launch(){
//设置服务端连接窗口的标题、位置、大小
        this.setTitle("远程控制客户端---连接窗口");

        this.setLocation(350, 200);

        this.setSize(500, 100);
//设置窗口始终置顶
        this.setAlwaysOnTop(true);

//设置一个层级面板
        JLayeredPane jlp  =new JLayeredPane();

        jlp.setLayout(new FlowLayout());

        iptf.setText("");
        porttf.setText("");
        idname.setText("");

        jlp.add(new JLabel("远程IP地址："));
        jlp.add(iptf);

        jlp.add(new JLabel("端口："));
        jlp.add(porttf);

        jlp.add(new JLabel("备注："));
        jlp.add(idname);

//监听按钮点击事件
        jb1.addMouseListener(new MouseAdapter(){

            public void mouseClicked(MouseEvent e) {
                try {
                    Controller.connect(iptf.getText(),Integer.parseInt(porttf.getText().trim()),idname.getText());
//                    if (iptf.getText()==null||iptf.getText())
                } catch (NumberFormatException e1) {

                    //端口号格式输入出错
                    JOptionPane.showMessageDialog(null,"IP地址或端口号输入出错，无法取得连接。。","提示",JOptionPane.ERROR_MESSAGE);
                    System.exit(0);
                    destroy();

                } catch (MyException e1) {
                    JOptionPane.showMessageDialog(null,e1.info,"提示",JOptionPane.ERROR_MESSAGE);

                }catch(Exception ed){

                    // "远程不允许被控，无法取得连接。。
                    JOptionPane.showMessageDialog(null,"远程不允许被控，无法取得连接。。","提示",JOptionPane.ERROR_MESSAGE);
                    System.exit(0);
                    destroy();

                }
                //连接成功
            }
        });

        jb2.addMouseListener(new MouseAdapter(){

            public void mouseClicked(MouseEvent e) {

                JOptionPane.showMessageDialog(null, "断开控制端窗口","提示",JOptionPane.INFORMATION_MESSAGE);

                System.exit(0);

                destroy();

            }

        });

        jlp.add(jb1);

        jlp.add(jb2);

        this.setLayeredPane(jlp);

        this.addWindowListener(new WindowAdapter(){


            public void windowClosing(WindowEvent e) {

                // 关闭窗口

                JOptionPane.showMessageDialog(null, "关闭连接窗口","提示",JOptionPane.INFORMATION_MESSAGE);

                System.exit(0);
            }

        });

        this.setVisible(true);

    }
    //启动方法
    public static void main(String[] args) {

        // TODO Auto-generated method stub

        new Server().launch();
    }



    public void destroy(){

        this.destroy();

    }

}