package com.sziit.G06.thread;

import java.awt.*;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.io.IOException;
import java.io.ObjectInputStream;

/*
 * 被控制端的事件回放(即回放控制端对被控制端的操作，包括：a.键盘事件(输入、删除)。
 *                                              b.鼠标事件(点击、移动、滑轮滚动)。
 * */
public class EventReadThread extends Thread {

    private ObjectInputStream ois;

    public EventReadThread(ObjectInputStream ois) {
//获得用于读取控制端事件对象的对象输入流，
        this.ois = ois;
    }

    @Override
   public void run() {
////循环读控制端事件并回放
       while (true) {
          try {
////读事件，回放事件
                Object eventobj = ois.readObject();
                InputEvent e = (InputEvent) eventobj;
                actionEvent(e);
            } catch (IOException | ClassNotFoundException e) {
                e.printStackTrace();
            }
        }
    }
//
//    //回放事件的方法
    private void actionEvent(InputEvent e) {
        Robot robot = null;
        try {
            robot = new Robot();
        } catch (AWTException e1) {
            e1.printStackTrace();
        }
//如果是键盘事件，回放处理键盘操作
        if (e instanceof KeyEvent) {
            KeyEvent ke = (KeyEvent) e;
            assert robot != null;
            if (e.getID() == KeyEvent.KEY_PRESSED) {
                robot.keyPress(ke.getKeyCode());
            }
            if (e.getID() == KeyEvent.KEY_RELEASED) {

                robot.keyRelease(ke.getKeyCode());
            }
        }
//如果是鼠标事件，回放处理鼠标操作
        if (e instanceof MouseEvent) {
            MouseEvent me = (MouseEvent) e;
            int type = me.getID();
            assert robot != null;
            if (type == MouseEvent.MOUSE_PRESSED) { //按下

                robot.mousePress(getMouseClick(me.getButton()));
            }
            if (type == MouseEvent.MOUSE_RELEASED) { //放开

                robot.mouseRelease(getMouseClick(me.getButton()));
            }
            if (type == MouseEvent.MOUSE_MOVED) { //移动

                robot.mouseMove(me.getX(), me.getY());
            }
            if (type == MouseEvent.MOUSE_DRAGGED) { //拖动

                robot.mouseMove(me.getX(), me.getY());
            }
            if (type == MouseEvent.MOUSE_WHEEL) { //滑轮滚动

                robot.mouseWheel(getMouseClick(me.getButton()));
            }
        }
    }


        /**
         * 根据发送的Mouse事件对象，转变为通用的Mouse按键代码
         *
         * @param button
         * @return
         */
    private int getMouseClick(int button) {
        if (button == MouseEvent.BUTTON1) {
            return InputEvent.BUTTON1_MASK;
        }
        if (button == MouseEvent.BUTTON2) {
            return InputEvent.BUTTON2_MASK;
        }
        if (button == MouseEvent.BUTTON3) {
            return InputEvent.BUTTON3_MASK;
        }
        return -1;
    }
}
